"use strict";
/**
 * # HttpClient
 *
 * HttpClient can be used to mock requests in your test-suite. There are
 * 4 functions: post(), get(), patch(), delete()
 *
 * **Before using, set HttpClient.url and HttpClient.port**
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpClient = void 0;
/**
 * HttpClient
 */
const environment_1 = require("../environment");
const request = require("request");
const http_client_response_1 = require("./http-client-response");
class HttpClient {
    constructor() {
        this.request = request;
        // Server URL
        this.url = 'http://localhost';
        // Server PORT
        this.port = environment_1.env.PORT;
    }
    /**
     * Send a POST http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    post(path, data, bearer = false, expect = [200, 201, 202, 204], customOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = {
                method: 'POST',
                url: `${this.url}:${this.port}${path}`,
                body: data,
                json: true
            };
            if (bearer) {
                options['auth'] = { bearer: `${bearer}` };
            }
            if (this.headers) {
                options['headers'] = this.headers;
            }
            if (customOptions) {
                options = Object.assign(options, customOptions);
            }
            return new Promise((accept, reject) => {
                this.request(options, (error, response, body) => {
                    if (error) {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                    else if (response) {
                        if (expect.includes(response.statusCode)) {
                            accept(new http_client_response_1.HttpClientResponse(response, body));
                        }
                        else {
                            reject(new http_client_response_1.HttpClientResponse(response, error));
                        }
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                    }
                });
            });
        });
    }
    /**
     * Send a GET http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    get(path, data = false, bearer = false, expect = [200, 202], customOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((accept, reject) => {
                let options = {
                    method: 'GET',
                    url: `${this.url}:${this.port}${path}`,
                    json: true,
                    qs: data
                };
                if (bearer) {
                    options['auth'] = { bearer: `${bearer}` };
                }
                if (this.headers) {
                    options['headers'] = this.headers;
                }
                if (customOptions) {
                    options = Object.assign(options, customOptions);
                }
                this.request(options, (error, response, body) => {
                    if (error) {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                    else if (response) {
                        if (expect.includes(response.statusCode)) {
                            accept(new http_client_response_1.HttpClientResponse(response, body));
                        }
                        else {
                            reject(new http_client_response_1.HttpClientResponse(response, error));
                        }
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                    }
                });
            });
        });
    }
    /**
     * Send a PATCH http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    patch(path, data, bearer = false, expect = [200, 201, 202, 204], customOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = {
                method: 'PATCH',
                url: `${this.url}:${this.port}${path}`,
                body: data,
                json: true
            };
            if (bearer) {
                options['auth'] = { bearer: `${bearer}` };
            }
            if (this.headers) {
                options['headers'] = this.headers;
            }
            if (customOptions) {
                options = Object.assign(options, customOptions);
            }
            return new Promise((accept, reject) => {
                this.request(options, (error, response, body) => {
                    if (error) {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                    else if (response) {
                        if (expect.includes(response.statusCode)) {
                            accept(new http_client_response_1.HttpClientResponse(response, body));
                        }
                        else {
                            reject(new http_client_response_1.HttpClientResponse(response, error));
                        }
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                    }
                });
            });
        });
    }
    /**
     * Send a DELETE http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    delete(path, bearer = false, expect = [200, 202, 204], customOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((accept, reject) => {
                let options = {
                    method: 'DELETE',
                    url: `${this.url}:${this.port}${path}`
                };
                if (bearer) {
                    options['auth'] = { bearer: `${bearer}` };
                }
                if (this.headers) {
                    options['headers'] = this.headers;
                }
                if (customOptions) {
                    options = Object.assign(options, customOptions);
                }
                this.request(options, (error, response, body) => {
                    if (error) {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                    else if (response) {
                        if (expect.includes(response.statusCode)) {
                            accept(new http_client_response_1.HttpClientResponse(response, body));
                        }
                        else {
                            reject(new http_client_response_1.HttpClientResponse(response, error));
                        }
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                    }
                });
            });
        });
    }
}
exports.HttpClient = HttpClient;
//# sourceMappingURL=http-client.js.map