"use strict";
/**
 * # JWT Tokens
 *
 * JWT Authorization headers are the default way to handle authentication.
 *
 * If you use the loginEndpoint and onlySelf/onlyAdmin guards, this will
 * be taken care of for you.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.jwtBearer = exports.JwtBearer = void 0;
const JWT = require("jsonwebtoken");
const environment_1 = require("./environment");
const btoa = require("btoa");
const atob = require("atob");
/**
 * JWT Bearer for authentication
 */
class JwtBearer {
    /**
     * Construct a JWT Bearer token
     * @param key Private key
     */
    constructor(key) {
        this.key = key !== null && key !== void 0 ? key : environment_1.env.JWT_KEY;
    }
    /**
     * Get the configured TTL for JWTs
     * @param isRefresh
     * @returns Returns the TTL in seconds
     */
    getTTLSeconds(isRefresh = false) {
        const ttlMin = isRefresh ?
            environment_1.env.JWT_REFRESH_TTL_MIN :
            environment_1.env.JWT_ACCESS_TTL_MIN;
        return ttlMin * 60;
    }
    /**
     * Get expiration of the token
     * @return Returns the expiration epoch time
     */
    getExpiration(isRefresh = false) {
        return (Date.now() +
            (this.getTTLSeconds(isRefresh) * 1000));
    }
    /**
     * Sign the token to the User
     * @param user User to sign the JWT for
     * @return Returns the signed, base64-encoded token
     */
    sign(user, isRefresh = false, data = {}) {
        const payload = Object.assign(data, {
            id: user.id,
            isRefresh: isRefresh
        });
        const jwt = JWT.sign(payload, this.key, {
            expiresIn: this.getTTLSeconds(isRefresh)
        });
        return btoa(jwt);
    }
    /**
     * Get a token from an HTTP Request
     * @param request Request to fetch the token from
     * @return Returns the token if successful, otherwise false
     */
    getToken(request) {
        const token = request.header('Authorization');
        if (token && token.includes('Bearer')) {
            return token.replace('Bearer ', '');
        }
        else {
            return false;
        }
    }
    /**
     * Verify the token
     * @param token Token to verify
     * @return Returns the token payload, or false if invalid
     */
    dryVerify(token) {
        try {
            return JWT.verify(atob(token), this.key);
        }
        catch (ex) {
            return false;
        }
    }
}
exports.JwtBearer = JwtBearer;
exports.jwtBearer = new JwtBearer();
//# sourceMappingURL=jwt-bearer.js.map