"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pointy = exports.PointyApi = void 0;
// Express
const express = require("express");
const bodyParser = require("body-parser");
// Pointy Core
const listen_1 = require("./utils/listen");
const database_1 = require("./database");
const http_client_1 = require("./http/http-client");
// Handlers
const responders_1 = require("./responders");
const handlers_1 = require("./handlers");
const bind_responders_1 = require("./utils/bind-responders");
const environment_1 = require("./environment");
const log_1 = require("./log");
// Base Models
const models_1 = require("./models");
/**
 * PointyAPI App Instance
 */
class PointyApi {
    // Initialize
    constructor() {
        // Express app
        this.app = express();
        // Core
        this.listen = listen_1.listen;
        this.http = new http_client_1.HttpClient();
        this.userType = models_1.ExampleUser;
        this.testmode = false;
        // Middleware
        // Handlers
        this.log = log_1.log.debug;
        this.error = handlers_1.errorHandler;
        // Responders
        this.conflictResponder = responders_1.conflictResponder;
        this.forbiddenResponder = responders_1.forbiddenResponder;
        this.goneResponder = responders_1.goneResponder;
        this.unauthorizedResponder = responders_1.unauthorizedResponder;
        this.validationResponder = responders_1.validationResponder;
        this.deleteResponder = responders_1.deleteResponder;
        this.getResponder = responders_1.getResponder;
        this.postResponder = responders_1.postResponder;
        this.patchResponder = responders_1.patchResponder;
        // Hooks
        this.before = (app) => { };
        this.ready = (app) => { };
        // Default db
        this.db = new database_1.PointyPostgres();
        if (process.argv.includes('testmode')) {
            this.testmode = true;
        }
        this.app.on('error', (error) => this.error(error));
    }
    // Ready check
    readycheck() {
        // Check pointy.userType
        if (!this.userType) {
            log_1.log.warn('[PointyAPI] pointy.userType has not been set!');
            return false;
        }
        // Check database connection
        if (!this.db || !this.db.conn) {
            log_1.log.warn('[PointyAPI] Database connection has not been established');
            return false;
        }
        // Check database entities
        if (!this.db.entities || !this.db.entities.length) {
            log_1.log.warn('[PointyAPI] Database does not contain any entities');
            return false;
        }
        // Check for database entity of BaseUser
        if (this.db.entities.includes(models_1.BaseUser)) {
            log_1.log.warn('[PointyAPI] BaseUser is not a valid database entity');
            return false;
        }
        // Checks passed
        return true;
    }
    // Start
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            // Set proper headers
            this.app.disable('x-powered-by');
            // Apply middleware functions
            this.app.use((request, response, next) => {
                request.userType = this.userType;
                request.joinMembers = [];
                // Bind the request & response to the responders
                (0, bind_responders_1.bindResponders)(this, request, response);
                next();
            });
            // Use body-parser
            this.app.use(bodyParser.json());
            this.app.use(bodyParser.urlencoded({ extended: true }));
            // Run before function
            yield this.before(this.app);
            // Ready check
            if (this.readycheck()) {
                // Server listen
                yield this.listen(this.app, environment_1.env.PORT);
                // Send IPC notification
                if ('send' in process && process.send) {
                    process.send('server-ready');
                }
                // Run ready callback
                this.ready(this.app);
            }
            else {
                process.exit();
            }
        });
    }
}
exports.PointyApi = PointyApi;
exports.pointy = new PointyApi();
//# sourceMappingURL=pointy-core.js.map