"use strict";
/**
 * # Set Model
 *
 * Set model sets the entity the request should use. You can also
 * set the route parameter (e.g. ':id'). This function should
 * be called in a middleware function BEFORE your endpoints.
 *
 * ## Auth Routes
 *
 * Auth routes must specify to `setModel` that it is an auth route. Do
 * this by passing `true` to the `isAuth` (fourth) parameter.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setModel = void 0;
const typeorm_1 = require("typeorm");
const middleware_1 = require("./middleware");
const utils_1 = require("./utils");
const query_validator_1 = require("./query-tools/query-validator");
/**
 * Set model type and load payload
 * @param request Request object
 * @param response Response object
 * @param model Model to set the request type to
 * @param isAuth If this is an authentication route.
 * 	Default is false.
 * 	Will run beforeLogin() and beforeLogout() posts instead of post/delete hook
 * @param identifier URL parameter name, for
 * 	example `/users/:id`. Default is `id`. Although this parameter
 * 	is optional, you **must** set it to the same string as in your routes.
 * @return Returns a Promise
 */
function setModel(request, response, model, isAuth = false, identifier = 'id') {
    return __awaiter(this, void 0, void 0, function* () {
        request.identifier = identifier;
        request.payloadType = model;
        request.payload = new model();
        request.repository = (0, typeorm_1.getRepository)(request.payloadType);
        // Substitute authenticated user as resource for auth router deletes
        if (isAuth && request.method === 'DELETE') {
            if (request.user) {
                request.params = request.user;
            }
            else {
                response.unauthorizedResponder('Not authenticated');
                return false;
            }
        }
        // Post loader
        if (request.method === 'POST') {
            // Load post array
            if (request.body instanceof Array) {
                for (let i = 0; i < request.body.length; i++) {
                    request.body[i] = Object.assign(new request.payloadType(), request.body[i]);
                    for (const key in request.body[i]) {
                        if (request.body[i][key] === undefined) {
                            delete request.body[i][key];
                        }
                        if (!(0, utils_1.isKeyInModel)(key, request.payload, response)) {
                            return false;
                        }
                    }
                }
            }
            else {
                // Load post object
                request.body = Object.assign(new request.payloadType(), request.body);
                for (const key in request.body) {
                    if (request.body[key] === undefined) {
                        delete request.body[key];
                    }
                    if (!(0, utils_1.isKeyInModel)(key, request.payload, response)) {
                        return false;
                    }
                }
            }
            // Run model hook
            if (!(yield (0, utils_1.runHook)(isAuth ? 'beforeLogin' : 'beforePost', request.body, request, response))) {
                return false;
            }
        }
        else if (request.method === 'GET') {
            // Get loader
            if (!(0, query_validator_1.queryValidator)(request, response)) {
                return false;
            }
            let getSuccess = true;
            request.payload = yield (0, middleware_1.getQuery)(request, response).catch((error) => {
                if (!response.headersSent) {
                    response.error(error);
                }
                getSuccess = false;
            });
            if (!getSuccess) {
                return false;
            }
        }
        else if (request.method === 'PATCH') {
            // Patch loader
            for (const key in request.body) {
                if (!(0, utils_1.isKeyInModel)(key, request.payload, response)) {
                    return false;
                }
            }
            // Load entity
            if (!(yield (0, middleware_1.loadEntity)(request, response))) {
                return false;
            }
            // Set model type
            request.body = Object.assign(new request.payloadType(), request.body);
            request.body = (0, utils_1.deleteUndefinedMembers)(request.body);
            // Run model hook
            if (!(yield (0, utils_1.runHook)('beforePatch', request.body, request, response))) {
                return false;
            }
        }
        else if (request.method === 'DELETE') {
            // Load entity
            if (!(yield (0, middleware_1.loadEntity)(request, response))) {
                return false;
            }
            // Delete loader
            // Run model hook
            if (!(yield (0, utils_1.runHook)(isAuth ? 'beforeLogout' : 'beforeDelete', request.payload, request, response))) {
                return false;
            }
        }
        return !response.headersSent;
    });
}
exports.setModel = setModel;
//# sourceMappingURL=set-model.js.map